# ANSI codes
ANSI_UP = "\u001b[A"
ANSI_DOWN = "\u001b[B"
ANSI_RIGHT = "\u001b[C"
ANSI_LEFT = "\u001b[D"

# Camera flags
FIXED = 1
CENTERED = 3
INCLUDE_SIZE = 5
CENTERED_AND_INCLUDE_SIZE = 8

__all__ = [
    "ANSI_UP",
    "ANSI_DOWN",
    "ANSI_RIGHT",
    "ANSI_LEFT",
    "FIXED",
    "CENTERED",
    "INCLUDE_SIZE",
    "CENTERED_AND_INCLUDE_SIZE"
]
